# Copyright (C) 2019 Red Hat Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import datetime
import json
import os
import sys
import unittest

exe = "tools_messages_tests"

if sys.version_info >= (3, 4):
    def set_fd_inheritable(fd):
        os.set_inheritable(fd, True)
else:
    def set_fd_inheritable(fd):
        pass


if sys.version_info >= (3, 0):
    def fdopen(fd, mode):
        return open(fd, mode)

    def isModuleInstalled(mod):
        import importlib.util
        return bool(importlib.util.find_spec(mod))
else:
    def fdopen(fd, mode):
        return os.fdopen(fd, mode)

    def isModuleInstalled(mod):
        import imp
        try:
            imp.find_module(mod)
            return True
        except ImportError:
            return False


def skipUnlessHasModule(mod):
    if not isModuleInstalled(mod):
        return unittest.skip("%s not available" % mod)
    return lambda func: func


def iterload(stream):
    dec = json.JSONDecoder()
    for line in stream:
        yield dec.raw_decode(line)


def loadJsonFromCommand(extraargs):
    r, w = os.pipe()
    set_fd_inheritable(r)
    r = fdopen(r, "r")
    set_fd_inheritable(w)
    w = fdopen(w, "w")
    pid = os.fork()
    if pid:
        w.close()
        l = list(iterload(r))
        l = [o[0] for o in l]
        r.close()
        return l
    else:
        r.close()
        args = ["tools_messages_tests",
                "--machine-readable=fd:%d" % w.fileno()] + extraargs
        os.execvp("./" + exe, args)


@skipUnlessHasModule('iso8601')
class TestParseToolsMessages(unittest.TestCase):
    def check_json(self, json, typ, msg):
        import iso8601
        # Check the type.
        jsontype = json.pop("type")
        self.assertEqual(jsontype, typ)
        # Check the message.
        jsonmsg = json.pop("message")
        self.assertEqual(jsonmsg, msg)
        # Check the timestamp.
        jsonts = json.pop("timestamp")
        dt = iso8601.parse_date(jsonts)
        now = datetime.datetime.now(dt.tzinfo)
        self.assertGreater(now, dt)
        # Check there are no more keys left (and thus not previously tested).
        self.assertEqual(len(json), 0)

    def test_messages(self):
        objects = loadJsonFromCommand([])
        self.assertEqual(len(objects), 4)
        self.check_json(objects[0], "message", "Starting")
        self.check_json(objects[1], "info", "An information message")
        self.check_json(objects[2], "warning", "Warning: message here")
        self.check_json(objects[3], "message", "Finishing")

    def test_error(self):
        objects = loadJsonFromCommand(["--error"])
        self.assertEqual(len(objects), 1)
        self.check_json(objects[0], "error", "Error!")


if __name__ == '__main__':
    unittest.main()



# -*- python -*-
# Copyright (C) 2018 Red Hat Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

# Fake ovirtsdk4 module used as a test harness.
# See v2v/test-v2v-o-rhv-upload.sh

class Error(Exception):
    pass


class NotFoundError(Error):
    pass


class Connection(object):
    def __init__(
            self,
            url=None,
            username=None,
            password=None,
            ca_file=None,
            log=None,
            insecure=False,
            debug=True,
    ):
        pass

    def close(self):
        pass

    def follow_link(self, objs):
        return objs

    def system_service(self):
        return SystemService()


class SystemService(object):
    def clusters_service(self):
        return ClustersService()

    def data_centers_service(self):
        return DataCentersService()

    def disks_service(self):
        return DisksService()

    def jobs_service(self):
        return JobsService()

    def image_transfers_service(self):
        return ImageTransfersService()

    def storage_domains_service(self):
        return StorageDomainsService()

    def vms_service(self):
        return VmsService()


class ClusterService(object):
    def get(self):
        return types.Cluster()


class ClustersService(object):
    def cluster_service(self, id):
        return ClusterService()


class DataCentersService(object):
    def list(self, search=None, case_sensitive=False):
        return [types.DataCenter()]


class DiskService(object):
    def __init__(self, disk_id):
        self._disk_id = disk_id

    def get(self):
        return types.Disk(id=self._disk_id)

    def remove(self):
        pass


class DisksService(object):
    def add(self, disk=None):
        disk.id = "756d81b0-d5c0-41bc-9bbe-b343c3fa3490"
        return disk

    def disk_service(self, disk_id):
        return DiskService(disk_id)


class JobsService(object):
    def list(self, search=None):
        return [types.Job()]


class ImageTransferService(object):
    def __init__(self):
        self._finalized = False

    def cancel(self):
        pass

    def get(self):
        if self._finalized:
            raise NotFoundError
        else:
            return types.ImageTransfer()

    def finalize(self):
        self._finalized = True


class ImageTransfersService(object):
    def add(self, transfer):
        return transfer

    def image_transfer_service(self, id):
        return ImageTransferService()


class StorageDomainsService(object):
    def list(self, search=None, case_sensitive=False):
        return [StorageDomain()]


class VmsService(object):
    def add(self, vm, query=None):
        return vm

    def list(self, search=None):
        return []
